@@ -7,6 +7,7 @@
 
 """Windows specific tests."""
 
+import datetime
 import errno
 import glob
 import os
@@ -16,29 +17,31 @@ import signal
 import subprocess
 import sys
 import time
-
-try:
-    import win32api  # requires "pip install pypiwin32" / "make setup-dev-env"
-    import win32con
-    import win32process
-    import wmi  # requires "pip install wmi" / "make setup-dev-env"
-except ImportError:
-    if os.name == 'nt':
-        raise
+import warnings
 
 import psutil
 from psutil import WINDOWS
-from psutil._compat import basestring
-from psutil._compat import callable
-from psutil._compat import PY3
 from psutil.tests import APPVEYOR
 from psutil.tests import get_test_subprocess
+from psutil.tests import HAS_BATTERY
 from psutil.tests import mock
+from psutil.tests import PY3
 from psutil.tests import reap_children
-from psutil.tests import retry_before_failing
-from psutil.tests import run_test_module_by_name
+from psutil.tests import retry_on_failure
+from psutil.tests import sh
 from psutil.tests import unittest
 
+with warnings.catch_warnings():
+    warnings.simplefilter("ignore")
+    try:
+        import win32api  # requires "pip install pypiwin32"
+        import win32con
+        import win32process
+        import wmi  # requires "pip install wmi" / "make setup-dev-env"
+    except ImportError:
+        if os.name == 'nt':
+            raise
+
 
 cext = psutil._psplatform.cext
 
@@ -65,14 +68,50 @@ def wrap_exceptions(fun):
 # ===================================================================
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
+class TestCpuAPIs(unittest.TestCase):
+
+    @unittest.skipIf('NUMBER_OF_PROCESSORS' not in os.environ,
+                     'NUMBER_OF_PROCESSORS env var is not available')
+    def test_cpu_count_vs_NUMBER_OF_PROCESSORS(self):
+        # Will likely fail on many-cores systems:
+        # https://stackoverflow.com/questions/31209256
+        num_cpus = int(os.environ['NUMBER_OF_PROCESSORS'])
+        self.assertEqual(num_cpus, psutil.cpu_count())
+
+    def test_cpu_count_vs_GetSystemInfo(self):
+        # Will likely fail on many-cores systems:
+        # https://stackoverflow.com/questions/31209256
+        sys_value = win32api.GetSystemInfo()[5]
+        psutil_value = psutil.cpu_count()
+        self.assertEqual(sys_value, psutil_value)
+
+    def test_cpu_count_logical_vs_wmi(self):
+        w = wmi.WMI()
+        proc = w.Win32_Processor()[0]
+        self.assertEqual(psutil.cpu_count(), proc.NumberOfLogicalProcessors)
+
+    def test_cpu_count_phys_vs_wmi(self):
+        w = wmi.WMI()
+        proc = w.Win32_Processor()[0]
+        self.assertEqual(psutil.cpu_count(logical=False), proc.NumberOfCores)
+
+    def test_cpu_count_vs_cpu_times(self):
+        self.assertEqual(psutil.cpu_count(),
+                         len(psutil.cpu_times(percpu=True)))
+
+    def test_cpu_freq(self):
+        w = wmi.WMI()
+        proc = w.Win32_Processor()[0]
+        self.assertEqual(proc.CurrentClockSpeed, psutil.cpu_freq().current)
+        self.assertEqual(proc.MaxClockSpeed, psutil.cpu_freq().max)
+
+
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestSystemAPIs(unittest.TestCase):
 
     def test_nic_names(self):
-        p = subprocess.Popen(['ipconfig', '/all'], stdout=subprocess.PIPE)
-        out = p.communicate()[0]
-        if PY3:
-            out = str(out, sys.stdout.encoding or sys.getfilesystemencoding())
+        out = sh('ipconfig /all')
         nics = psutil.net_io_counters(pernic=True).keys()
         for nic in nics:
             if "pseudo-interface" in nic.replace(' ', '-').lower():
@@ -81,23 +120,6 @@ class TestSystemAPIs(unittest.TestCase):
                 self.fail(
                     "%r nic wasn't found in 'ipconfig /all' output" % nic)
 
-    @unittest.skipUnless('NUMBER_OF_PROCESSORS' in os.environ,
-                         'NUMBER_OF_PROCESSORS env var is not available')
-    def test_cpu_count(self):
-        num_cpus = int(os.environ['NUMBER_OF_PROCESSORS'])
-        self.assertEqual(num_cpus, psutil.cpu_count())
-
-    def test_cpu_count_2(self):
-        sys_value = win32api.GetSystemInfo()[5]
-        psutil_value = psutil.cpu_count()
-        self.assertEqual(sys_value, psutil_value)
-
-    def test_cpu_freq(self):
-        w = wmi.WMI()
-        proc = w.Win32_Processor()[0]
-        self.assertEqual(proc.CurrentClockSpeed, psutil.cpu_freq().current)
-        self.assertEqual(proc.MaxClockSpeed, psutil.cpu_freq().max)
-
     def test_total_phymem(self):
         w = wmi.WMI().Win32_ComputerSystem()[0]
         self.assertEqual(int(w.TotalPhysicalMemory),
@@ -116,7 +138,7 @@ class TestSystemAPIs(unittest.TestCase):
 
     # Note: this test is not very reliable
     @unittest.skipIf(APPVEYOR, "test not relieable on appveyor")
-    @retry_before_failing()
+    @retry_on_failure()
     def test_pids(self):
         # Note: this test might fail if the OS is starting/killing
         # other processes in the meantime
@@ -125,7 +147,7 @@ class TestSystemAPIs(unittest.TestCase):
         psutil_pids = set(psutil.pids())
         self.assertEqual(wmi_pids, psutil_pids)
 
-    @retry_before_failing()
+    @retry_on_failure()
     def test_disks(self):
         ps_parts = psutil.disk_partitions(all=True)
         wmi_parts = wmi.WMI().Win32_LogicalDisk()
@@ -135,6 +157,8 @@ class TestSystemAPIs(unittest.TestCase):
                     if not ps_part.mountpoint:
                         # this is usually a CD-ROM with no disk inserted
                         break
+                    if 'cdrom' in ps_part.opts:
+                        break
                     try:
                         usage = psutil.disk_usage(ps_part.mountpoint)
                     except OSError as err:
@@ -156,6 +180,8 @@ class TestSystemAPIs(unittest.TestCase):
 
     def test_disk_usage(self):
         for disk in psutil.disk_partitions():
+            if 'cdrom' in disk.opts:
+                continue
             sys_value = win32api.GetDiskFreeSpaceEx(disk.mountpoint)
             psutil_value = psutil.disk_usage(disk.mountpoint)
             self.assertAlmostEqual(sys_value[0], psutil_value.free,
@@ -182,39 +208,60 @@ class TestSystemAPIs(unittest.TestCase):
         self.assertTrue(ps_names & wmi_names,
                         "no common entries in %s, %s" % (ps_names, wmi_names))
 
+    def test_boot_time(self):
+        wmi_os = wmi.WMI().Win32_OperatingSystem()
+        wmi_btime_str = wmi_os[0].LastBootUpTime.split('.')[0]
+        wmi_btime_dt = datetime.datetime.strptime(
+            wmi_btime_str, "%Y%m%d%H%M%S")
+        psutil_dt = datetime.datetime.fromtimestamp(psutil.boot_time())
+        diff = abs((wmi_btime_dt - psutil_dt).total_seconds())
+        self.assertLessEqual(diff, 1)
+
+    def test_boot_time_fluctuation(self):
+        # https://github.com/giampaolo/psutil/issues/1007
+        with mock.patch('psutil._pswindows.cext.boot_time', return_value=5):
+            self.assertEqual(psutil.boot_time(), 5)
+        with mock.patch('psutil._pswindows.cext.boot_time', return_value=4):
+            self.assertEqual(psutil.boot_time(), 5)
+        with mock.patch('psutil._pswindows.cext.boot_time', return_value=6):
+            self.assertEqual(psutil.boot_time(), 5)
+        with mock.patch('psutil._pswindows.cext.boot_time', return_value=333):
+            self.assertEqual(psutil.boot_time(), 333)
+
 
 # ===================================================================
 # sensors_battery()
 # ===================================================================
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestSensorsBattery(unittest.TestCase):
 
-    def test_percent(self):
-        w = wmi.WMI()
-        battery_psutil = psutil.sensors_battery()
-        if battery_psutil is None:
-            with self.assertRaises(IndexError):
-                w.query('select * from Win32_Battery')[0]
-        else:
-            battery_wmi = w.query('select * from Win32_Battery')[0]
-            if battery_psutil is None:
-                self.assertNot(battery_wmi.EstimatedChargeRemaining)
-                return
-
-            self.assertAlmostEqual(
-                battery_psutil.percent, battery_wmi.EstimatedChargeRemaining,
-                delta=1)
-            self.assertEqual(
-                battery_psutil.power_plugged, battery_wmi.BatteryStatus == 1)
-
-    def test_battery_present(self):
+    def test_has_battery(self):
         if win32api.GetPwrCapabilities()['SystemBatteriesPresent']:
             self.assertIsNotNone(psutil.sensors_battery())
         else:
             self.assertIsNone(psutil.sensors_battery())
 
+    @unittest.skipIf(not HAS_BATTERY, "no battery")
+    def test_percent(self):
+        w = wmi.WMI()
+        battery_wmi = w.query('select * from Win32_Battery')[0]
+        battery_psutil = psutil.sensors_battery()
+        self.assertAlmostEqual(
+            battery_psutil.percent, battery_wmi.EstimatedChargeRemaining,
+            delta=1)
+
+    @unittest.skipIf(not HAS_BATTERY, "no battery")
+    def test_power_plugged(self):
+        w = wmi.WMI()
+        battery_wmi = w.query('select * from Win32_Battery')[0]
+        battery_psutil = psutil.sensors_battery()
+        # Status codes:
+        # https://msdn.microsoft.com/en-us/library/aa394074(v=vs.85).aspx
+        self.assertEqual(battery_psutil.power_plugged,
+                         battery_wmi.BatteryStatus == 2)
+
     def test_emulate_no_battery(self):
         with mock.patch("psutil._pswindows.cext.sensors_battery",
                         return_value=(0, 128, 0, 0)) as m:
@@ -248,7 +295,7 @@ class TestSensorsBattery(unittest.TestCase):
 # ===================================================================
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestProcess(unittest.TestCase):
 
     @classmethod
@@ -284,12 +331,19 @@ class TestProcess(unittest.TestCase):
         p = psutil.Process(self.pid)
         self.assertRaises(ValueError, p.send_signal, signal.SIGINT)
 
-    def test_exe(self):
+    def test_exe_and_name(self):
         for p in psutil.process_iter():
+            # On Windows name() is never supposed to raise AccessDenied,
+            # see https://github.com/giampaolo/psutil/issues/627
             try:
-                self.assertEqual(os.path.basename(p.exe()), p.name())
-            except psutil.Error:
+                name = p.name()
+            except psutil.NoSuchProcess:
                 pass
+            else:
+                try:
+                    self.assertEqual(os.path.basename(p.exe()), name)
+                except psutil.Error:
+                    continue
 
     def test_num_handles_increment(self):
         p = psutil.Process(os.getpid())
@@ -318,7 +372,7 @@ class TestProcess(unittest.TestCase):
             if name.startswith('_') \
                     or name in ('terminate', 'kill', 'suspend', 'resume',
                                 'nice', 'send_signal', 'wait', 'children',
-                                'as_dict'):
+                                'as_dict', 'memory_info_ex'):
                 continue
             else:
                 try:
@@ -337,17 +391,8 @@ class TestProcess(unittest.TestCase):
         if failures:
             self.fail('\n' + '\n'.join(failures))
 
-    def test_name_always_available(self):
-        # On Windows name() is never supposed to raise AccessDenied,
-        # see https://github.com/giampaolo/psutil/issues/627
-        for p in psutil.process_iter():
-            try:
-                p.name()
-            except psutil.NoSuchProcess:
-                pass
-
-    @unittest.skipUnless(sys.version_info >= (2, 7),
-                         "CTRL_* signals not supported")
+    @unittest.skipIf(not sys.version_info >= (2, 7),
+                     "CTRL_* signals not supported")
     def test_ctrl_signals(self):
         p = psutil.Process(get_test_subprocess().pid)
         p.send_signal(signal.CTRL_C_EVENT)
@@ -359,20 +404,9 @@ class TestProcess(unittest.TestCase):
         self.assertRaises(psutil.NoSuchProcess,
                           p.send_signal, signal.CTRL_BREAK_EVENT)
 
-    def test_compare_name_exe(self):
-        for p in psutil.process_iter():
-            try:
-                a = os.path.basename(p.exe())
-                b = p.name()
-            except (psutil.NoSuchProcess, psutil.AccessDenied):
-                pass
-            else:
-                self.assertEqual(a, b)
-
     def test_username(self):
-        sys_value = win32api.GetUserName()
-        psutil_value = psutil.Process().username()
-        self.assertEqual(sys_value, psutil_value.split('\\')[1])
+        self.assertEqual(psutil.Process().username(),
+                         win32api.GetUserNameEx(win32con.NameSamCompatible))
 
     def test_cmdline(self):
         sys_value = re.sub(' +', ' ', win32api.GetCommandLine()).strip()
@@ -475,18 +509,18 @@ class TestProcess(unittest.TestCase):
         import ctypes.wintypes
         PROCESS_QUERY_INFORMATION = 0x400
         handle = ctypes.windll.kernel32.OpenProcess(
-            PROCESS_QUERY_INFORMATION, 0, os.getpid())
+            PROCESS_QUERY_INFORMATION, 0, self.pid)
         self.addCleanup(ctypes.windll.kernel32.CloseHandle, handle)
+
         hndcnt = ctypes.wintypes.DWORD()
         ctypes.windll.kernel32.GetProcessHandleCount(
             handle, ctypes.byref(hndcnt))
         sys_value = hndcnt.value
-        psutil_value = psutil.Process().num_handles()
-        ctypes.windll.kernel32.CloseHandle(handle)
-        self.assertEqual(psutil_value, sys_value + 1)
+        psutil_value = psutil.Process(self.pid).num_handles()
+        self.assertEqual(psutil_value, sys_value)
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestProcessWMI(unittest.TestCase):
     """Compare Process API results with WMI."""
 
@@ -552,7 +586,7 @@ class TestProcessWMI(unittest.TestCase):
         self.assertEqual(wmic_create, psutil_create)
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestDualProcessImplementation(unittest.TestCase):
     """
     Certain APIs on Windows have 2 internal implementations, one
@@ -626,11 +660,26 @@ class TestDualProcessImplementation(unittest.TestCase):
         num_handles = psutil.Process(self.pid).num_handles()
         with mock.patch("psutil._psplatform.cext.proc_num_handles",
                         side_effect=OSError(errno.EPERM, "msg")) as fun:
-            psutil.Process(self.pid).num_handles() == num_handles
+            self.assertEqual(psutil.Process(self.pid).num_handles(),
+                             num_handles)
             assert fun.called
 
+    def test_cmdline(self):
+        from psutil._pswindows import convert_oserror
+        for pid in psutil.pids():
+            try:
+                a = cext.proc_cmdline(pid, use_peb=True)
+                b = cext.proc_cmdline(pid, use_peb=False)
+            except OSError as err:
+                err = convert_oserror(err)
+                if not isinstance(err, (psutil.AccessDenied,
+                                        psutil.NoSuchProcess)):
+                    raise
+            else:
+                self.assertEqual(a, b)
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class RemoteProcessTestCase(unittest.TestCase):
     """Certain functions require calling ReadProcessMemory.
     This trivially works when called on the current process.
@@ -715,9 +764,10 @@ class RemoteProcessTestCase(unittest.TestCase):
 
     def test_environ_64(self):
         p = psutil.Process(self.proc64.pid)
-        e = p.environ()
-        self.assertIn("THINK_OF_A_NUMBER", e)
-        self.assertEquals(e["THINK_OF_A_NUMBER"], str(os.getpid()))
+        try:
+            p.environ()
+        except psutil.AccessDenied:
+            pass
 
 
 # ===================================================================
@@ -725,7 +775,7 @@ class RemoteProcessTestCase(unittest.TestCase):
 # ===================================================================
 
 
-@unittest.skipUnless(WINDOWS, "WINDOWS only")
+@unittest.skipIf(not WINDOWS, "WINDOWS only")
 class TestServices(unittest.TestCase):
 
     def test_win_service_iter(self):
@@ -754,19 +804,19 @@ class TestServices(unittest.TestCase):
         ])
         for serv in psutil.win_service_iter():
             data = serv.as_dict()
-            self.assertIsInstance(data['name'], basestring)
+            self.assertIsInstance(data['name'], str)
             self.assertNotEqual(data['name'].strip(), "")
-            self.assertIsInstance(data['display_name'], basestring)
-            self.assertIsInstance(data['username'], basestring)
+            self.assertIsInstance(data['display_name'], str)
+            self.assertIsInstance(data['username'], str)
             self.assertIn(data['status'], valid_statuses)
             if data['pid'] is not None:
                 psutil.Process(data['pid'])
-            self.assertIsInstance(data['binpath'], basestring)
-            self.assertIsInstance(data['username'], basestring)
-            self.assertIsInstance(data['start_type'], basestring)
+            self.assertIsInstance(data['binpath'], str)
+            self.assertIsInstance(data['username'], str)
+            self.assertIsInstance(data['start_type'], str)
             self.assertIn(data['start_type'], valid_start_types)
             self.assertIn(data['status'], valid_statuses)
-            self.assertIsInstance(data['description'], basestring)
+            self.assertIsInstance(data['description'], str)
             pid = serv.pid()
             if pid is not None:
                 p = psutil.Process(pid)
@@ -777,16 +827,22 @@ class TestServices(unittest.TestCase):
             self.assertEqual(serv, s)
 
     def test_win_service_get(self):
-        name = next(psutil.win_service_iter()).name()
+        ERROR_SERVICE_DOES_NOT_EXIST = \
+            psutil._psplatform.cext.ERROR_SERVICE_DOES_NOT_EXIST
+        ERROR_ACCESS_DENIED = psutil._psplatform.cext.ERROR_ACCESS_DENIED
 
+        name = next(psutil.win_service_iter()).name()
         with self.assertRaises(psutil.NoSuchProcess) as cm:
             psutil.win_service_get(name + '???')
         self.assertEqual(cm.exception.name, name + '???')
 
         # test NoSuchProcess
         service = psutil.win_service_get(name)
-        exc = WindowsError(
-            psutil._psplatform.cext.ERROR_SERVICE_DOES_NOT_EXIST, "")
+        if PY3:
+            args = (0, "msg", 0, ERROR_SERVICE_DOES_NOT_EXIST)
+        else:
+            args = (ERROR_SERVICE_DOES_NOT_EXIST, "msg")
+        exc = WindowsError(*args)
         with mock.patch("psutil._psplatform.cext.winservice_query_status",
                         side_effect=exc):
             self.assertRaises(psutil.NoSuchProcess, service.status)
@@ -795,8 +851,11 @@ class TestServices(unittest.TestCase):
             self.assertRaises(psutil.NoSuchProcess, service.username)
 
         # test AccessDenied
-        exc = WindowsError(
-            psutil._psplatform.cext.ERROR_ACCESS_DENIED, "")
+        if PY3:
+            args = (0, "msg", 0, ERROR_ACCESS_DENIED)
+        else:
+            args = (ERROR_ACCESS_DENIED, "msg")
+        exc = WindowsError(*args)
         with mock.patch("psutil._psplatform.cext.winservice_query_status",
                         side_effect=exc):
             self.assertRaises(psutil.AccessDenied, service.status)
@@ -812,4 +871,5 @@ class TestServices(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    run_test_module_by_name(__file__)
+    from psutil.tests.runner import run
+    run(__file__)
